#version 430
#extension GL_ARB_separate_shader_objects : enable










struct SComputeInput
{
	uvec3 GlobalThreadID; uvec3 LocalThreadID; uvec3 GroupID;};

layout( std140) uniform UData
{
	uint Size;
	uint KernelSize;
	float HalfAngle;
} Data;
 uniform sampler2D InputTexture;
 uniform writeonly image2D OutputTexture;










float area(float x, float y)
{
	return atan(x * y, sqrt(x * x + y * y + 1.0f));
}

void ComputeMain( in SComputeInput  cInput  )
{
	uint size =  Data.Size ;
	uint kernelSize =  Data.KernelSize ;
	float halfAngle =  Data.HalfAngle ;

	if (cInput.GlobalThreadID.x >= size || cInput.GlobalThreadID.y >= size) return;

	vec3 sum = vec3(0, 0, 0);
	float totalWeight = 0.0f;


	float texSize =  float(size * 2 + kernelSize * 2) ;


	vec2 uvTexel = (vec2(0.5, 0.5) +  vec2(cInput.GlobalThreadID.xy) ) /  float(size) ;
	vec3 r0 = normalize(vec3(uvTexel * 2.0f - 1.0f, 1.0f));

	int range =  int(kernelSize)  + 1;

	for (int x = -range; x < range; ++x)
	{
		for (int y = -range; y < range; ++y)
		{
			vec2 os = vec2(x, y) + 0.5;
			vec2 coords =  vec2(cInput.GlobalThreadID.xy)  * 2.0f +  float(kernelSize)  + os + 1.0f;
			vec2 nCoords = (coords - texSize * 0.5f) /  float(size) ;
			vec2 halfTex = vec2(0.5, 0.5) / (2. *  float(size) );
			vec2 delta = max(abs(nCoords) - vec2(1., 1.), vec2(0.,0.));


			vec3 r1;
			{
				vec2 nCoordsClamp = clamp(nCoords, vec2(-1., -1.) + halfTex, vec2(1., 1.) - halfTex);
				r1 = normalize(vec3(nCoordsClamp, 1.0f - delta.x - delta.y));
			}


			float angle = acos(dot(r1, r0));
			if ((angle > halfAngle) ||
				(delta.x > 0.0f && delta.y > 0.0f))
				continue;


			float sa;
			{
				vec2 nCoordsMirror = nCoords - 2. * (nCoords - clamp(nCoords, vec2(-1., -1.), vec2(1., 1.)));
				vec2 nCoordsP = nCoordsMirror + halfTex;
				vec2 nCoordsN = nCoordsMirror - halfTex;
				sa = area(nCoordsN.x, nCoordsN.y) - area(nCoordsN.x, nCoordsP.y) - area(nCoordsP.x, nCoordsN.y) + area(nCoordsP.x, nCoordsP.y);
			}


			float w =  clamp(cos(0.5f * 3.14159f * angle / halfAngle), 0.0, 1.0) ;

			vec3 sampledCol =  texelFetch(InputTexture, (ivec3(ivec2(coords), 0)).xy, (ivec3(ivec2(coords), 0)).z) .rgb;

			w *= sa;
			sum += w*sampledCol;
			totalWeight += w;
		}
	}
	sum /= totalWeight;

	vec4 col = vec4(0, 0, 0, 1);
	col.rgb = sum;

	imageStore(OutputTexture, ivec2(cInput.GlobalThreadID.xy), col) ;
}


layout(local_size_x = 32, local_size_y = 32, local_size_z = 1) in;
void main()
{
	SComputeInput cInput;
	cInput.GlobalThreadID = gl_GlobalInvocationID;
	cInput.LocalThreadID = gl_LocalInvocationID;
	cInput.GroupID = gl_WorkGroupID;
	ComputeMain(cInput);
}
