#version 330
#extension GL_ARB_separate_shader_objects : enable





layout(location = 0) smooth in vec2 _fin_fragTexCoord;

struct SFragmentInput
{
	vec2 fragTexCoord;
	bool IsFrontFace;
};

layout(location = 0) out vec4 _fout_outColor;

struct SFragmentOutput
{
	vec4 outColor;
};

layout( std140) uniform UInfo
{
	vec4 Top;
	vec4 Bottom;
	vec4 Origin;
	mat4 UserToRHY;
	mat4 InvMat;
	vec4 EnvironmentMapColor;
	float EnvironmentMapMipLevel;
	uint EnvironmentMapVisible;
} Info;
 uniform samplerCube EnvironmentMapSampler;
layout( std140) uniform UEnvironmentMapInfo
{
	mat2 Rotation;
} EnvironmentMapInfo;



void FragmentMain( in SFragmentInput  fInput,  out SFragmentOutput  fOutput  )
{
	vec4 topColor =  Info.Top ;
	vec4 bottomColor =  Info.Bottom ;
	mat4 invMat =  Info.InvMat ;

	vec4 wsp =  ((invMat) * (vec4((fInput.fragTexCoord * 2.0f - 1.0f), 1.f, 1.f))) ;
	vec3 dirVec = normalize(wsp.xyz/wsp.w -  Info.Origin .xyz);
	vec3 frameDir =  (( Info.UserToRHY ) * (vec4(dirVec, 0.0f))) .xyz;
	float up = dot(frameDir, vec3(0, 1, 0));

	if (int( Info.EnvironmentMapVisible ) != 0)
	{
		vec4 envMapColor =  Info.EnvironmentMapColor ;
		frameDir = normalize(frameDir);
		frameDir.xz =  (( EnvironmentMapInfo.Rotation ) * (frameDir.xz)) ;
		fOutput.outColor = vec4( textureLod(EnvironmentMapSampler, frameDir, Info.EnvironmentMapMipLevel ) .rgb * envMapColor.rgb, 1);
	}
	else
		fOutput.outColor = mix(bottomColor, topColor, (1.0f + up) * 0.5f);
}

void main()
{
	SFragmentInput fInput;
	SFragmentOutput fOutput;

	fInput.fragTexCoord = _fin_fragTexCoord;
	fInput.IsFrontFace = gl_FrontFacing;
	FragmentMain(fInput, fOutput);
	_fout_outColor = fOutput.outColor;
}
