#version 330
#extension GL_ARB_separate_shader_objects : enable





layout(location = 0) smooth in vec3 _fin_fragNormal;
layout(location = 1) smooth in vec4 _fin_fragTangent;
layout(location = 2) smooth in vec2 _fin_fragTexCoord;
layout(location = 3) smooth in vec4 _fin_fragWorldPosition;
layout(location = 4) smooth in vec4 _fin_fragColor;

struct SFragmentInput
{
	vec3 fragNormal;
	vec4 fragTangent;
	vec2 fragTexCoord;
	vec4 fragWorldPosition;
	vec4 fragColor;
	bool IsFrontFace;
};

layout(location = 0) out vec4 _fout_outDiffuse;
layout(location = 1) out float _fout_outDepth;
layout(location = 2) out vec4 _fout_outEmissive;
layout(location = 3) out vec4 _fout_outNormalSpec;

struct SFragmentOutput
{
	vec4 outDiffuse;
	float outDepth;
	vec4 outEmissive;
	vec4 outNormalSpec;
};

layout( std140) uniform USceneInfo
{
	mat4 ViewProj;
	mat4 View;
	mat4 Proj;
	mat4 InvView;
	mat4 InvViewProj;
	mat4 UserToLHZ;
	mat4 LHZToUser;
	mat4 BillboardingView;
	mat4 PackNormalView;
	mat4 UnpackNormalView;
	vec4 SideVector;
	vec4 DepthVector;
	vec2 ZBufferLimits;
	vec2 ViewportSize;
	float Handedness;
} SceneInfo;
layout( std140) uniform UMeshFragmentConstant
{
	float Roughness;
	float Metalness;
} MeshFragmentConstant;
 uniform sampler2D DiffuseTex;





vec2 PackNormalSpheremap(vec3 normal  )
{
	normal = normalize(normal);
	normal =  (( SceneInfo.PackNormalView ) * (vec4(normal, 0.0f))) .xyz;
	float f = sqrt(8.0f * normal.z + 8.0f);
	return normal.xy / f + 0.5f;
}

void FragmentMain( in SFragmentInput  fInput,  out SFragmentOutput  fOutput  )
{


	vec4 diffuseColor;


	diffuseColor =  texture(DiffuseTex, fInput.fragTexCoord) ;
	diffuseColor *= fInput.fragColor;




	float roughness =  MeshFragmentConstant.Roughness ;
	float metalness =  MeshFragmentConstant.Metalness ;
	vec3 normal = fInput.fragNormal;


	fOutput.outDiffuse = diffuseColor;
	fOutput.outNormalSpec = vec4(PackNormalSpheremap(normal  ), roughness, metalness);
	fOutput.outDepth = fInput.fragWorldPosition.z / fInput.fragWorldPosition.w;
	fOutput.outEmissive = vec4(0,0,0,0);
}


void main()
{
	SFragmentInput fInput;
	SFragmentOutput fOutput;

	fInput.fragNormal = _fin_fragNormal;
	fInput.fragTangent = _fin_fragTangent;
	fInput.fragTexCoord = _fin_fragTexCoord;
	fInput.fragWorldPosition = _fin_fragWorldPosition;
	fInput.fragColor = _fin_fragColor;
	fInput.IsFrontFace = gl_FrontFacing;
	FragmentMain(fInput, fOutput);
	_fout_outDiffuse = fOutput.outDiffuse;
	_fout_outDepth = fOutput.outDepth;
	_fout_outEmissive = fOutput.outEmissive;
	_fout_outNormalSpec = fOutput.outNormalSpec;
}
