#version 330
#extension GL_ARB_separate_shader_objects : enable





layout(location = 0) smooth in vec2 _fin_fragTexCoord;

struct SFragmentInput
{
	vec2 fragTexCoord;
	bool IsFrontFace;
};

layout(location = 0) out vec4 _fout_outColor;

struct SFragmentOutput
{
	vec4 outColor;
};

 uniform sampler2D Texture;
 uniform sampler2D LookUp;

layout( std140) uniform UBlurInfo
{
	vec2 ContextSize;
	vec2 Direction;
	vec4 SubUV;
} BlurInfo;



vec4 sampleTextureForBlur( sampler2D toSample , vec2 uv, vec4 subUV  )
{
	vec2 fragUV = clamp(uv, 0.0f, 1.0f) * subUV.zw + subUV.xy;
	return  texture(toSample, fragUV) ;
}

vec4 blur5(vec2 uv, float radius  )
{
	vec2 direction =  BlurInfo.Direction  * radius;
	vec2 resolution =  BlurInfo.ContextSize ;
	vec4 subUV =  BlurInfo.SubUV ;
	vec4 color =  vec4(0,0,0,0) ;
	vec2 off1 = (1.3333333333333333 * direction) / resolution;
	vec4 sampleMid = sampleTextureForBlur( Texture , uv, subUV  );
	color += sampleMid * 0.29411764705882354;
	color += sampleTextureForBlur( Texture , uv + off1, subUV  ) * 0.35294117647058826;
	color += sampleTextureForBlur( Texture , uv - off1, subUV  ) * 0.35294117647058826;
	return vec4(color.rgb, sampleMid.a);
}

vec4 blur9(vec2 uv, float radius  )
{
	vec2 direction =  BlurInfo.Direction  * radius;
	vec2 resolution =  BlurInfo.ContextSize ;
	vec4 subUV =  BlurInfo.SubUV ;
	vec4 color =  vec4(0,0,0,0) ;
	vec2 off1 = (1.3846153846 * direction) / resolution;
	vec2 off2 = (3.2307692308 * direction) / resolution;
	vec4 sampleMid = sampleTextureForBlur( Texture , uv, subUV  );
	color += sampleMid * 0.2270270270;
	color += sampleTextureForBlur( Texture , uv + off1, subUV  ) * 0.3162162162;
	color += sampleTextureForBlur( Texture , uv - off1, subUV  ) * 0.3162162162;
	color += sampleTextureForBlur( Texture , uv + off2, subUV  ) * 0.0702702703;
	color += sampleTextureForBlur( Texture , uv - off2, subUV  ) * 0.0702702703;
	return vec4(color.rgb, sampleMid.a);
}

vec4 blur13(vec2 uv, float radius  )
{
	vec2 direction =  BlurInfo.Direction  * radius;
	vec2 resolution =  BlurInfo.ContextSize ;
	vec4 subUV =  BlurInfo.SubUV ;
	vec4 color =  vec4(0,0,0,0) ;
	vec2 off1 = (1.411764705882353 * direction) / resolution;
	vec2 off2 = (3.2941176470588234 * direction) / resolution;
	vec2 off3 = (5.176470588235294 * direction) / resolution;
	vec4 sampleMid = sampleTextureForBlur( Texture , uv, subUV  );
	color += sampleMid * 0.1964825501511404;
	color += sampleTextureForBlur( Texture , uv + off1, subUV  ) * 0.2969069646728344;
	color += sampleTextureForBlur( Texture , uv - off1, subUV  ) * 0.2969069646728344;
	color += sampleTextureForBlur( Texture , uv + off2, subUV  ) * 0.09447039785044732;
	color += sampleTextureForBlur( Texture , uv - off2, subUV  ) * 0.09447039785044732;
	color += sampleTextureForBlur( Texture , uv + off3, subUV  ) * 0.010381362401148057;
	color += sampleTextureForBlur( Texture , uv - off3, subUV  ) * 0.010381362401148057;
	return vec4(color.rgb, sampleMid.a);
}

void FragmentMain( in SFragmentInput  fInput,  out SFragmentOutput  fOutput  )
{
	float blurRadius = 1.0f;



	blurRadius = min( texture(LookUp, fInput.fragTexCoord) .z, 1.0f);







	fOutput.outColor = blur9(fInput.fragTexCoord, blurRadius  );
}


void main()
{
	SFragmentInput fInput;
	SFragmentOutput fOutput;

	fInput.fragTexCoord = _fin_fragTexCoord;
	fInput.IsFrontFace = gl_FrontFacing;
	FragmentMain(fInput, fOutput);
	_fout_outColor = fOutput.outColor;
}
