#version 430
#extension GL_ARB_separate_shader_objects : enable










struct SComputeInput
{
	uvec3 GlobalThreadID; uvec3 LocalThreadID; uvec3 GroupID;};

layout(std430 ) readonly buffer BInCounts
{
	uint InCounts[];
} InCounts;
layout(std430 ) readonly buffer BInKeys
{
	uint InKeys[];
} InKeys;
layout(std430 ) readonly buffer BInIndirection
{
	uint InIndirection[];
} InIndirection;
layout(std430 ) buffer BOutKeys
{
	uint OutKeys[];
} OutKeys;
layout(std430 ) buffer BOutIndirection
{
	uint OutIndirection[];
} OutIndirection;

layout( std140) uniform UInfo
{
	uint CurrentBit;
	uint GroupCount;
} Info;

shared uint tempSharedMem[6144];
shared uint groupOffsets[16];
shared uint localDigitOffsets[16];







void ComputeMain( in SComputeInput  cInput  )
{
	const uint globalThreadID = cInput.GlobalThreadID.x;
	const uint localThreadID = cInput.LocalThreadID.x;
	const uint groupID = cInput.GroupID.x;

	const uint currentBit =  Info.CurrentBit  *  4 ;
	const uint blocki = groupID *  128  *  24  + localThreadID;




	uvec2 keys[ 24 ];
	uint ranks[ 24 ];



	for (uint item = 0; item <  24 ; ++item)
	{

		const uvec2 fullKey =  uvec2( InKeys.InKeys [ ((blocki + item * 128 ) * 2) ], InKeys.InKeys [( ((blocki + item * 128 ) * 2) ) + 1]) ;
		tempSharedMem [(localThreadID + item *  128 )] = fullKey.x;
		tempSharedMem [(localThreadID + item *  128 ) +  128  *  24 ] = fullKey.y;
	}

	groupMemoryBarrier(); barrier(); ;




	for (uint item = 0; item <  24 ; ++item)
	{

		keys[item] = uvec2( tempSharedMem [(localThreadID *  24  + item)],  tempSharedMem [(localThreadID *  24  + item) +  128  *  24 ]);
	}

	groupMemoryBarrier(); barrier(); ;


	{
		const uint ai = localThreadID *  (1 << ( 4 )) ;


		for (uint k = 0; k <  (1 << ( 4 )) ; ++k)
		{
			tempSharedMem [ ((ai + k) + ((ai + k) >> 4)) ] = 0;
		}
	}

	groupMemoryBarrier(); barrier(); ;



	for (uint item = 0; item <  24 ; ++item)
	{

		const uvec2 fullKey = keys[item];
		const uint keyBits = ((currentBit < 32 ? fullKey.y : fullKey.x) >> (currentBit % 32)) & ( (1 << ( 4 ))  - 1);
		ranks[item] =  tempSharedMem [ ((localThreadID * (1 << ( 4 )) + keyBits) + ((localThreadID * (1 << ( 4 )) + keyBits) >> 4)) ];
		tempSharedMem [ ((localThreadID * (1 << ( 4 )) + keyBits) + ((localThreadID * (1 << ( 4 )) + keyBits) >> 4)) ] += 1;
	}


	{
		uint stride = 1;






		for (uint d =  128  / 2; d > 0; d /= 2)
		{
			const uint numBinsPerThread = max( uint(1) , d *  (1 << ( 4 ))  /  128 );
			const uint numThreadsPerHist =  (1 << ( 4 ))  / numBinsPerThread;

			groupMemoryBarrier(); barrier(); ;
			if (localThreadID < d *  (1 << ( 4 )) )
			{
				const uint localLane = localThreadID / (numThreadsPerHist);
				const uint localOffset = localThreadID & (numThreadsPerHist - 1);
				const uint ai = ((localLane * 2 + 1) * stride - 1) *  (1 << ( 4 )) ;
				const uint bi = ((localLane * 2 + 2) * stride - 1) *  (1 << ( 4 )) ;

				for (uint k = 0; k < numBinsPerThread; ++k)
				{
					tempSharedMem [ ((bi + localOffset + k * numThreadsPerHist) + ((bi + localOffset + k * numThreadsPerHist) >> 4)) ] +=  tempSharedMem [ ((ai + localOffset + k * numThreadsPerHist) + ((ai + localOffset + k * numThreadsPerHist) >> 4)) ];
				}
			}
			stride *= 2;
		}


		for (uint d = 1; d <  128 ; d *= 2)
		{
			const uint numBinsPerThread = max( uint(1) , d *  (1 << ( 4 ))  /  128 );
			const uint numThreadsPerHist =  (1 << ( 4 ))  / numBinsPerThread;
			stride /= 2;
			groupMemoryBarrier(); barrier(); ;
			if (localThreadID > (numThreadsPerHist - 1) && localThreadID < d *  (1 << ( 4 )) )
			{
				const uint localLane = localThreadID / (numThreadsPerHist);
				const uint localOffset = localThreadID & (numThreadsPerHist - 1);
				const uint ai = ((localLane * 2 + 0) * stride - 1) *  (1 << ( 4 )) ;
				const uint bi = ((localLane * 2 + 1) * stride - 1) *  (1 << ( 4 )) ;

				for (uint k = 0; k < numBinsPerThread; ++k)
				{
					tempSharedMem [ ((bi + localOffset + k * numThreadsPerHist) + ((bi + localOffset + k * numThreadsPerHist) >> 4)) ] +=  tempSharedMem [ ((ai + localOffset + k * numThreadsPerHist) + ((ai + localOffset + k * numThreadsPerHist) >> 4)) ];
				}
			}
		}
	}

	groupMemoryBarrier(); barrier(); ;


	{

		if (localThreadID <  (1 << ( 4 )) )
		{
			localDigitOffsets [localThreadID] =  tempSharedMem [ ((( 128 - 1) * (1 << ( 4 )) + localThreadID) + ((( 128 - 1) * (1 << ( 4 )) + localThreadID) >> 4)) ];
		}


		uint stride = 1;

		for (uint d =  (1 << ( 4 ))  / 2; d > 0; d /= 2)
		{
			groupMemoryBarrier(); barrier(); ;
			if (localThreadID < d)
			{
				const uint ai = ((localThreadID * 2 + 1) * stride - 1);
				const uint bi = ((localThreadID * 2 + 2) * stride - 1);

				localDigitOffsets [bi] +=  localDigitOffsets [ai];
			}
			stride *= 2;
		}

		groupMemoryBarrier(); barrier(); ;

		if (localThreadID == 0)
		{
			localDigitOffsets [ (1 << ( 4 ))  - 1] = 0;
		}


		for (uint d = 1; d <  (1 << ( 4 )) ; d *= 2)
		{
			stride /= 2;
			groupMemoryBarrier(); barrier(); ;
			if (localThreadID < d)
			{
				const uint ai = (localThreadID * 2 + 1) * stride - 1;
				const uint bi = (localThreadID * 2 + 2) * stride - 1;
				const uint t =  localDigitOffsets [ai];
				localDigitOffsets [ai] =  localDigitOffsets [bi];
				localDigitOffsets [bi] += t;
			}
		}
	}

	groupMemoryBarrier(); barrier(); ;


	if (localThreadID <  (1 << ( 4 )) )
	{
		groupOffsets [localThreadID] =  InCounts.InCounts [ (groupID * (1 << ( 4 )) + localThreadID) ]  -  localDigitOffsets [localThreadID];
	}

	groupMemoryBarrier(); barrier(); ;



	for (uint item = 0; item <  24 ; ++item)
	{

		const uvec2 fullKey = keys[item];
		const uint keyBits = ((currentBit < 32 ? fullKey.y : fullKey.x) >> (currentBit % 32)) & ( (1 << ( 4 ))  - 1);
		uint localOffset =  localDigitOffsets [keyBits];

		if (localThreadID != 0)
			localOffset +=  tempSharedMem [ ((((localThreadID - 1) * (1 << ( 4 )) ) + keyBits) + ((((localThreadID - 1) * (1 << ( 4 )) ) + keyBits) >> 4)) ];

		ranks[item] += localOffset;
	}

	groupMemoryBarrier(); barrier(); ;





	for (uint item = 0; item <  24 ; ++item)
	{

		tempSharedMem [ranks[item]] = keys[item].x;
		tempSharedMem [ranks[item] +  128  *  24 ] = keys[item].y;
	}

	groupMemoryBarrier(); barrier(); ;

	uint localGroupOffsets[ 24 ];



	for (uint item = 0; item <  24 ; ++item)
	{

		const uvec2 fullKey = uvec2( tempSharedMem [(localThreadID + item *  128 )],  tempSharedMem [(localThreadID + item *  128 ) +  128  *  24 ]);
		const uint keyBits = ((currentBit < 32 ? fullKey.y : fullKey.x) >> (currentBit % 32)) & ( (1 << ( 4 ))  - 1);
		const uint groupOffset =  groupOffsets [keyBits];

		localGroupOffsets[item] = groupOffset;

		const uint finalOffset = localThreadID + item *  128  + groupOffset;


		OutKeys.OutKeys [ (finalOffset * 2) ] = (fullKey).x; OutKeys.OutKeys [( (finalOffset * 2) ) + 1] = (fullKey).y ;
	}

	groupMemoryBarrier(); barrier(); ;



	uint values[ 24 ];



	for (uint item = 0; item <  24 ; ++item)
	{
		const uint inValues =  InIndirection.InIndirection [ (blocki + item * 128 ) ] ;
		tempSharedMem [localThreadID + item *  128 ] = inValues;
	}

	groupMemoryBarrier(); barrier(); ;


	for (uint item = 0; item <  24 ; ++item)
	{
		values[item] =  tempSharedMem [localThreadID *  24  + item];
	}

	groupMemoryBarrier(); barrier(); ;


	for (uint item = 0; item <  24 ; ++item)
	{
		tempSharedMem [ranks[item]] = values[item];
	}

	groupMemoryBarrier(); barrier(); ;



	for (uint item = 0; item <  24 ; ++item)
	{
		const uint inValues =  tempSharedMem [localThreadID + item *  128 ];
		const uint finalOffset = localThreadID + item *  128  + localGroupOffsets[item];
		OutIndirection.OutIndirection [ (finalOffset) ] = inValues; ;
	}
}

layout(local_size_x = 128, local_size_y = 1, local_size_z = 1) in;
void main()
{
	SComputeInput cInput;
	cInput.GlobalThreadID = gl_GlobalInvocationID;
	cInput.LocalThreadID = gl_LocalInvocationID;
	cInput.GroupID = gl_WorkGroupID;
	ComputeMain(cInput);
}
